import { invoke } from "@tauri-apps/api"
import { getCurrent, WindowManager } from "@tauri-apps/api/window"
import { defineStore } from "pinia"
import { computed, ref, watch } from "vue"
import { useRoute } from "vue-router"
import router from "../router"

const webview: WindowManager = getCurrent()

export const useReaderStore = defineStore('reader', () => {

    const loadingChapter = ref(true)

    const route = useRoute()

    const timestamp = ref(Date.now())

    // Chapter List

    const chapterList = ref<string[]>([])

    const getChapterList = async () => {
        timestamp.value = Date.now()
        chapterList.value = await invoke('get_chapter_list_by_title', { title: route.params.title })
        chapterList.value.push()
        if (route.params.chapter === '0') router.push({
          path: `/read/${route.params.title}/${chapterList.value[0]}/0`
        })
    }

    // Change chapters

    const indexOfChapter = () => chapterList.value.findIndex((chapter) => chapter === route.params.chapter)

    const nextChapter = () => {
        const index = indexOfChapter()
        const nextChapter = chapterList.value[index + 1]
        if (!nextChapter) return
        router.push(`/read/${route.params.title}/${nextChapter}/0`)
    }
    
    const prevChapter = () => {
        const index = indexOfChapter()
        const prevChapter = chapterList.value[index - 1]
        if (!prevChapter) return
        router.push(`/read/${route.params.title}/${prevChapter}/-1`)
    }

    // Title

    const updateTitle = async () => {
        webview.setTitle(`${route.params.title} - ${route.params.chapter}`)
    }      

    // Chapter Data 

    const chapterDataDefault = {
        path: '',
        images: [],
    }

    const chapterData = ref(chapterDataDefault)

    const resetChapterData = () => {
        chapterData.value = chapterDataDefault
    }

    const updateChapterData = async () => {

        if (['-1', '0'].includes(route.params.chapter as string)) return

        loadingChapter.value = true

        chapterData.value = await invoke('get_chapter_by_title', {
            title: route.params.title,
            chapter: route.params.chapter
        })

        updateTitle()
        updateDiscordRP()

        loadingChapter.value = false
    }

    const updateDiscordRP = async () => {

        if (!route.params.title || !route.params.chapter) return

        // if (!(await webview.isVisible())) return

        await invoke('discord_set_activity', {
            details: "Reading manga",
            state: `${route.params.title} - ${route.params.chapter}`,
            timestamp: timestamp.value,
            image: "logo",
        })
    }

    const currentChapter = computed(() => route.params.chapter)
    
    watch(currentChapter, async (value, oldValue) => {
        if (value === oldValue) return
        if (!route.params.title || !route.params.chapter) return
        updateChapterData()
    }, { immediate: true })

    watch(route, async() => {

        // console.log(oldValue?.params?.chapter)
        // console.log(value?.params?.chapter)

        if (!route.params.title || !route.params.chapter) return

        invoke('set_reader_state', {
            title: route.params.title as string || "",
            chapter: route.params.chapter as string || "",
            slide: parseInt(route.params.slide as string) || 0,
        })

        // if (value?.params?.title !== oldValue?.params?.title) {
        //     await getChapterList()
        //     // await updateChapterData()
        //     return
        // }

        // if (value?.params?.chapter !== oldValue?.params?.chapter) {
        //     await updateChapterData()
        // }

    }, { immediate: true })

    // Slider 

    const changeSlideRoute = (slide: Number) => {
        router.push(`/read/${route.params.title}/${route.params.chapter}/${slide}`)
    }

    const setChapter = (chapter: String) => {
        router.push(`/read/${route.params.title}/${chapter}/0`)
    }

    const push = async (url: string) => {
        await router.push(url)
    }

    // Return

    return { chapterList, loadingChapter, getChapterList, updateChapterData, resetChapterData, nextChapter, prevChapter, chapterData, changeSlideRoute, setChapter, updateDiscordRP, push, timestamp }
})